extends Area2D

export var check_areas = false
export var check_solids = true
export var checked_areas = []

var ignored_bodies = []
var overlapping_bodies = []

onready var parent = get_node("..")

# Set up connections.
func _ready():
	if check_solids:
		connect("body_enter", self, "_on_Checker_body_enter")
		connect("body_exit", self, "_on_Checker_body_exit")
	if check_areas:
		connect("area_enter", self, "_on_Checker_area_enter")
		connect("area_exit", self, "_on_Checker_area_exit")

# Track all non-parent bodies which enter us.
func _on_Checker_body_enter( body ):
	if body != parent and ignored_bodies.find(body) == -1:
		if overlapping_bodies.find(body) == -1:
			overlapping_bodies.append(body)

# Forget about all bodies which leave us.
func _on_Checker_body_exit( body ):
	while overlapping_bodies.find(body) != -1:
		overlapping_bodies.erase(body)

# Track any areas which enter our range and are in specific groups.
func _on_Checker_area_enter( area ):
	for group in checked_areas:
		if area.is_in_group(group) and ignored_bodies.find(area) == -1:
			if overlapping_bodies.find(area) == -1:
				overlapping_bodies.append(area)

# Forget about all areas which leave us.
func _on_Checker_area_exit( area ):
	while overlapping_bodies.find(area) != -1:
		overlapping_bodies.erase(area)

# Return true iff we're overlapping at least one (non-parent) body/area.
func is_overlapping():
	# Don't count bodies which should be temporarily ignored.
	for body in overlapping_bodies:
		if parent.temporarily_ignore_collisions.find(body) == -1:
			return true
	return false
