# Class storing balance info about all monsters as well as player-oriented state about them.
extends Node

const Drop = preload("res://Items/Drop.gd")
const Enemy = preload("res://Actors/Enemies/Enemy.gd")
const FileUtil = preload("res://Util/FileUtil.gd")

var data = {}

# For now initialize everything anew. Eventually, persist.
func _init():
	for file in FileUtil.find_files_of_type("res://Actors/Enemies", true, "tscn"):
		_add_beast(file)

# Add a new Beast to the Bestiary. Only works if the path is a scene which extends Enemy.
func _add_beast(path):
	var beast = load(path).instance()
	if not beast extends Enemy:
		return
	var entry = Entry.new()
	entry.name = beast.name
	entry.description = beast.description
	entry.mass = beast.get_mass()
	entry.hp = beast.health
	entry.xp = beast.xp
	entry.dmg = beast.get_node("Hitbox").damage
	entry.resistances = beast.resistances
	entry.weaknesses = beast.weaknesses
	if (beast.drop_paths != null):
		for i in range(beast.drop_paths.size()):
			var drop = Drop.new()
			drop.path = beast.drop_paths[i]
			drop.rate = beast.drop_rates[i]
			entry.drops.append(drop)
	data[path] = entry

# Beastiary entry for each monster scene.
class Entry:
	var name = null
	var description = null
	var mass = null
	var hp = null
	var xp = null
	var dmg = null
	var resistances = null
	var weaknesses = null
	var drops = []
	var encountered = 0
	var killed = 0
