extends "res://Actors/Player/States/PlayerState.gd"

const Ducking = preload("res://Actors/Player/States/Ducking.gd")
const Floating = preload("res://Actors/Player/States/Floating.gd")
const Jumping = preload("res://Actors/Player/States/Jumping.gd")
const SlideKick = preload("res://Actors/Player/Weapons/SlideKick/SlideKick.tscn")
const Standing = preload("res://Actors/Player/States/Standing.gd")

const DOWNWARD_ANGLE = PI / 4.0
const SPEED_MODIFIER = 1.5

var _bounce = false
var _direction = 0
var _slide_kick_weapon = null
var _speed = null

func get_name():
	return "DiveKicking"

func enter(player):
	.enter(player)
	_speed = player.horizontal_speed * player.walking_multiplier * player.running_multiplier * SPEED_MODIFIER
	if Input.is_action_pressed("player_left") or Input.is_action_pressed("player_right"):
		_direction = player.direction
	# Deform the SlideKick weapon a bit for our purposes.
	_slide_kick_weapon = SlideKick.instance()
	_slide_kick_weapon.set_pos(Vector2(13, 35))
	_slide_kick_weapon.set_rot(PI / 2)
	_slide_kick_weapon.set_scale(Vector2(1, 2))
	var pos = _slide_kick_weapon.get_pos()
	if player.direction == -1:
		_slide_kick_weapon.set_pos(Vector2(-1 * pos.x, pos.y))
	_slide_kick_weapon.connect("area_enter", self, "_bounce")
	player.add_child(_slide_kick_weapon)
	_slide_kick_weapon.activate()

func _bounce(area):
	if area.is_in_group("Grabbable"):
		_bounce = true

func _fixed_process(delta):
	# Bounce off grabbables.
	if _bounce == true:
		var jumping = Jumping.new()
		jumping.forced_jump = true
		return jumping
	# Stop if we've landed.
	if player.is_in_water():
		return Floating.new()
	if player.is_grounded():
		if Input.is_action_pressed("player_duck"):
			return Ducking.new()
		return Standing.new()
	var delta_speed_slice = _speed * delta
	player.move_by(Vector2(delta_speed_slice * _direction, delta_speed_slice * SPEED_MODIFIER))

func exit():
	player.remove_child(_slide_kick_weapon)
