extends "res://Actors/Player/States/PlayerState.gd"

const Dodging = preload("res://Actors/Player/States/Dodging.gd")
const Falling = preload("res://Actors/Player/States/Falling.gd")
const Jumping = preload("res://Actors/Player/States/Jumping.gd")
const SlideKicking = preload("res://Actors/Player/States/SlideKicking.gd")
const Standing = preload("res://Actors/Player/States/Standing.gd")

const DUCKING_HEIGHT_MULTIPLIER = 0.5
const HIGH_JUMP_CHARGE_DELAY = 1.0 # s
const UP = Vector2(0, 1) # One-way collision detection is backward.
const DOWN = Vector2(0, -1)

var _duck_time = 0
var _floor_checker = null
var _will_high_jump = false

func get_name():
	return "Ducking"

func enter(player):
	.enter(player)
	_floor_checker = player.get_node("FloorChecker")
	player.set_shape_to_ducking()
	player.has_dodge = player.can_dodge
	player.has_extra_jump = player.can_double_jump

func _input(event):
	# Slide kicking and high jumping both come from the same button.
	if player.should_jump(event) and not player.is_attacking():
		player.get_tree().set_input_as_handled()
		# High jump if we have a charge.
		if _duck_time > HIGH_JUMP_CHARGE_DELAY:
			_will_high_jump = true
			return Jumping.new()
		# Drop through a platform if it had one-way collisions and we're not holding a direction.
		if not Input.is_action_pressed("player_left") and not Input.is_action_pressed("player_right"):
			for body in _floor_checker.overlapping_bodies:
				if body extends PhysicsBody2D:
					if body.get_one_way_collision_direction() == UP:
						player.temporarily_ignore_collisions.append(body)
						body.set_one_way_collision_direction(DOWN)
						return Falling.new()
		# Slide kick if we're not blocked.
		if not player.is_roof_blocked() and not player.is_face_checking_a_wall():
			return SlideKicking.new()
		if player.is_roof_blocked():
			return SlideKicking.new()
		# Regular jump.
		return Jumping.new()
	# Dodging.
	if (event.type == InputEvent.KEY or event.type == InputEvent.JOYSTICK_BUTTON) and event.is_pressed() and not event.is_echo():
		if player.has_dodge:
			if event.is_action("player_dodge_directional"):
				player.get_tree().set_input_as_handled()
				return Dodging.new()
			if event.is_action("player_dodge"):
				player.get_tree().set_input_as_handled()
				var dodging = Dodging.new()
				dodging.dodge_backward = true
				return dodging

func _fixed_process(delta):
	# Change directions.
	if Input.is_action_pressed("player_right") and not player.is_attacking():
		player.direction = 1
	elif Input.is_action_pressed("player_left") and not player.is_attacking():
		player.direction = -1
	# Stop ducking.
	if not player.is_grounded():
		return Falling.new()
	if not Input.is_action_pressed("player_duck") and not player.is_roof_blocked():
		return Standing.new()
	# Charge the high jump.
	if _duck_time > HIGH_JUMP_CHARGE_DELAY:
		if player.jump_height_multiplier < player.MAX_JUMP_HEIGHT_MULTIPLIER:
			player.jump_height_multiplier = player.jump_height_multiplier + delta
			if player.jump_height_multiplier > player.MAX_JUMP_HEIGHT_MULTIPLIER:
				player.jump_height_multiplier = player.MAX_JUMP_HEIGHT_MULTIPLIER
	_duck_time = _duck_time + delta

func exit():
	player.set_shape_to_default()
	if not _will_high_jump:
		player.jump_height_multiplier = 1.0
