extends "res://Actors/Player/States/PlayerState.gd"

const DiveKicking = preload("res://Actors/Player/States/DiveKicking.gd")
const Dodging = preload("res://Actors/Player/States/Dodging.gd")
const Ducking = preload("res://Actors/Player/States/Ducking.gd")
const Floating = preload("res://Actors/Player/States/Floating.gd")
const Jumping = preload("res://Actors/Player/States/Jumping.gd")
const Standing = preload("res://Actors/Player/States/Standing.gd")
const WallJumping = preload("res://Actors/Player/States/WallJumping.gd")

const CREST_TIME = 0.5 # s
const GLIDING_MULTIPLIER = 0.5
const MAX_FALL_SPEED = 375.0 # pixels/sec
const UP = Vector2(0, 1)

var _player_hitbox = null;
var _speed_mod = null
var _timer = CREST_TIME
var _y_velocity = 0.0

func get_name():
	return "Falling"

func enter(player):
	.enter(player)
	_player_hitbox = player.get_node("Hitbox")
	_speed_mod = player.agility / 100.0

func _input(event):
	if player.should_jump(event):
		# Dive kicks.
		if Input.is_action_pressed("player_duck"):
			player.get_tree().set_input_as_handled()
			return DiveKicking.new()
		# Wall Jumps.
		var wall_jump_direction = player.should_wall_jump_to()
		if wall_jump_direction != 0:
			player.get_tree().set_input_as_handled()
			var state = WallJumping.new()
			state.direction = wall_jump_direction
			return state
		var could_grab = player.could_grab()
		# Wall jump off grabbables.
		if not could_grab.empty():
			var direction = 1
			if could_grab[0].get_global_pos() >= player.get_global_pos():
				direction = -1
			var state = WallJumping.new()
			state.direction = direction
			return state
		# Double Jump
		if player.has_extra_jump:
			player.has_extra_jump = false
			player.get_tree().set_input_as_handled()
			return Jumping.new()
	if (event.type == InputEvent.KEY or event.type == InputEvent.JOYSTICK_BUTTON) and event.is_pressed() and not event.is_echo() and event.is_action("player_dodge_directional"):
		if player.has_dodge:
			player.get_tree().set_input_as_handled()
			return Dodging.new()

# Fall increasingly fast until another state is triggered.
func _fixed_process(delta):
	# Stop falling if we hit the ground.
	if player.is_grounded():
		if Input.is_action_pressed("player_duck"):
			return Ducking.new()
		return Standing.new()
	# Splashdown if we hit water.
	if player.is_in_water():
		return Floating.new()
	# Accelerate downward, which is delayed if we can glide.
	var glide_mod = 1
	if player.can_glide and Input.is_action_pressed("player_alternate"):
		glide_mod = GLIDING_MULTIPLIER
	_timer = _timer - (delta * glide_mod)
	if _timer > 0:
		_y_velocity = ((CREST_TIME - _timer) / CREST_TIME) * MAX_FALL_SPEED * _speed_mod * delta * glide_mod
	else:
		_y_velocity = MAX_FALL_SPEED * _speed_mod * delta * glide_mod
	var x_velocity = player.get_horizontal_movement() * delta
	player.move_by(Vector2(x_velocity, _y_velocity))

func exit():
	# Reset any one-way collision exceptions, since they were only set by Ducking prior to entering us.
	for body in player.temporarily_ignore_collisions:
		body.set_one_way_collision_direction(UP)
	player.temporarily_ignore_collisions.clear()
	# Potentially jump-cancel attacks.
	if player.attack_animation_player.is_playing():
		# Teleport the arms back to neutral. A bit kludgey.
		player.animation_player.play("Standing", 0)
		player.cancel_attack()
