extends "res://Actors/Player/States/PlayerState.gd"

const Ducking = preload("res://Actors/Player/States/Ducking.gd")
const Falling = preload("res://Actors/Player/States/Falling.gd")
const Floating = preload("res://Actors/Player/States/Floating.gd")
const Standing = preload("res://Actors/Player/States/Standing.gd")

const ACCELERATION = 640 # pixels/sec at straight up or down
const DECELERATION = ACCELERATION * 1.25
# Angle required to maintain velocity.
const MAINTENANCE_ANGLE = -0.26179938779914943653855361527329 # -PI / 12
const PI_OVER_TWO = 1.5707963267948966192313216916398
const VELOCITY_CUTOFF = 32 # pixels/sec

var _angle = MAINTENANCE_ANGLE
var _direction = null
var _sprite = null
var _velocity = null
var _wall_checker_L = null
var _wall_checker_R = null

func get_name():
	return "Flying"

func enter(player):
	.enter(player)
	player.set_shape_to_ducking()
	_velocity = player.horizontal_speed * player.walking_multiplier * player.running_multiplier
	_direction = player.direction
	_sprite = player.get_node("Sprite")
	_wall_checker_L = player.get_node("WallCheckerL")
	_wall_checker_R = player.get_node("WallCheckerR")

# Aiming for Mario 64 wing cap effects here.
func _fixed_process(delta):
	# Adjust angle.
	if _should_dive():
		_angle = _angle - (PI_OVER_TWO * delta)
	elif _should_rise():
		_angle = _angle + (PI_OVER_TWO * delta)
	if _angle > PI_OVER_TWO:
		_angle = PI_OVER_TWO
	if _angle < -PI_OVER_TWO:
		_angle = -PI_OVER_TWO
	_sprite.set_rot(_angle * player.direction)
	# Adjust velocities.
	var velocity_step = ACCELERATION * abs(sin(_angle)) * delta
	if _angle > MAINTENANCE_ANGLE:
		_velocity = _velocity - velocity_step
	elif _angle < MAINTENANCE_ANGLE:
		_velocity = _velocity + velocity_step
	# Decelerate harshly if we're grounded or hit a wall.
	if player.is_grounded():
		_angle = 0
		_velocity = _velocity - (DECELERATION * delta)
	if player.is_face_checking_a_wall() or player.is_in_water():
		_velocity = _velocity - (DECELERATION * delta)
	# Only stop flying if we come to a stop.
	if _velocity <= VELOCITY_CUTOFF:
		if player.is_in_water():
			return Floating.new()
		if player.is_grounded():
			if Input.is_action_pressed("player_duck"):
				return Ducking.new()
			return Standing.new()
		return Falling.new()
	player.move_by(Vector2(_velocity * _direction * cos(_angle) * delta, (-1) * _velocity * sin(_angle) * delta))
 
# Return true iff the player should dive.
func _should_dive():
	if _direction == 1 and Input.is_action_pressed("ui_right"):
		return true
	if _direction == -1 and Input.is_action_pressed("ui_left"):
		return true
	if Input.get_joy_axis(0, JOY_AXIS_1) >= player.JOYSTICK_THRESHOLD:
		return true
	return false

# Return true iff the player should rise.
func _should_rise():
	if _direction == 1 and Input.is_action_pressed("ui_left"):
		return true
	if _direction == -1 and Input.is_action_pressed("ui_right"):
		return true
	if Input.get_joy_axis(0, JOY_AXIS_1) <= -1 * player.JOYSTICK_THRESHOLD:
		return true
	return false

func exit():
	player.set_shape_to_default()
	_sprite.set_rot(0)
