extends "res://Actors/Player/States/PlayerState.gd"

const DiveKicking = preload("res://Actors/Player/States/DiveKicking.gd")
const Dodging = preload("res://Actors/Player/States/Dodging.gd")
const Falling = preload("res://Actors/Player/States/Falling.gd")
const Flying = preload("res://Actors/Player/States/Flying.gd")

const CREST_HEIGHT_FRACTION = 0.25
const CREST_TIME = 0.05 # s
const JUMP_TIME = 0.45 # s

var forced_jump = false # Require that the player jumps to their full height, regardless of button-pressing.

var _jump_height_multiplier = null
var _jump_time_remaining = JUMP_TIME

func get_name():
	return "Jumping"

func enter(player):
	.enter(player)
	# Take note of and consume any built-up high-jump multiplier.
	_jump_height_multiplier = player.jump_height_multiplier
	player.jump_height_multiplier = 1.0

func _input(event):
	if (event.type == InputEvent.KEY or event.type == InputEvent.JOYSTICK_BUTTON) and event.is_pressed() and not event.is_echo():
		if event.is_action("player_dodge_directional"):
			if player.has_dodge:
				player.get_tree().set_input_as_handled()
				return Dodging.new()
		# Dive kick out of forced jumps.
		if event.is_action("player_jump") and Input.is_action_pressed("player_duck"):
			player.get_tree().set_input_as_handled()
			return DiveKicking.new()

# Go upward mostly linearly until the crest or button release.
func _fixed_process(delta):
	_jump_time_remaining = _jump_time_remaining - delta
	# Stop going up if the button has been released.
	if not (Input.is_action_pressed("player_jump") or forced_jump == true) and _jump_time_remaining > 0:
		_jump_time_remaining = 0
	# Determine slice of motion based on jump_height and time fraction.
	var y_velocity = player.jump_height * (delta / JUMP_TIME) * _jump_height_multiplier
	# Reduced percentage for slightly rounded transitions.
	if _jump_time_remaining <= 0:
		y_velocity = y_velocity * CREST_HEIGHT_FRACTION * ((CREST_TIME + _jump_time_remaining) / CREST_TIME)
	# Fall or Fly, depending.
	if y_velocity > 0:
		if _jump_height_multiplier > 1.0 and Input.is_action_pressed("player_alternate"):
			return Flying.new()
		return Falling.new()
	var x_velocity = player.get_horizontal_movement() * delta
	player.move_by(Vector2(x_velocity, y_velocity))
