extends "res://Actors/Player/States/PlayerState.gd"

const Falling = preload("res://Actors/Player/States/Falling.gd")

const CREST_HEIGHT_FRACTION = 0.25
const CREST_TIME = 0.1 # s
const DIVE_TIME = 0.9 # s
const DISTANCE_MOD = 4

var _direction = null
var _dive_time_remaining = DIVE_TIME
var _wall_checker_l = null
var _wall_checker_r = null

func enter(player):
	_direction = -1 * player.direction # Inverted since we copied the jumping logic, which goes upward.
	_wall_checker_l = player.get_node("WallCheckerL")
	_wall_checker_r = player.get_node("WallCheckerR")
	.enter(player)

func get_name():
	return "KamikazeDiving"

# Very similar to DragonPunching.gd (but horizontal).
func _fixed_process(delta):
	_dive_time_remaining = _dive_time_remaining - delta
	# Determine slice of motion based on jump_height and time fraction.
	var x_velocity = player.jump_height * (delta / DIVE_TIME) * DISTANCE_MOD
	# Reduced percentage for slightly rounded transitions.
	if _dive_time_remaining <= 0:
		x_velocity = x_velocity * CREST_HEIGHT_FRACTION * ((CREST_TIME + _dive_time_remaining) / CREST_TIME)
	# Once our x_velocity dies out, start Falling.
	if x_velocity > 0:
		return Falling.new()
	# Or if we've hit a thing.
	if (_direction == -1 and _wall_checker_r.is_overlapping()) or (_direction == 1 and _wall_checker_l.is_overlapping()):
		return Falling.new()
	# Only horizontal movement while KamikazeDiving.
	player.move_by(Vector2(_direction * x_velocity, 0))

# Go through enemies while KamikazeDiving.
func hit_enemy(enemy):
	pass

func exit():
	# Don't let the default transition occur or we'll end up with a funny scale.
	player.get_node("AnimationPlayer").play("Falling", 0)
	player.get_node("Sprite/Torso").set_scale(Vector2(1, 1))
	player.cancel_attack()
