extends "res://Actors/Player/States/PlayerState.gd"

const Ducking = preload("res://Actors/Player/States/Ducking.gd")
const Falling = preload("res://Actors/Player/States/Falling.gd")
const Floating = preload("res://Actors/Player/States/Floating.gd")
const SlideKick = preload("res://Actors/Player/Weapons/SlideKick/SlideKick.tscn")
const Standing = preload("res://Actors/Player/States/Standing.gd")

const DECELERATION_DURATION = 0.25 # s
const MAX_DURATION = 0.4 # s
const PI_OVER_TWO = 1.5707963267948966192313216916398
const SPEED_MODIFIER = 1.75

var _average_y_speed = null
var _has_landed = false
var _initial_speed = null
var _rising = true
var _speed = null
var _stop_timer = 0
var _timer = 0
var _slide_kick_weapon = null

func get_name():
	return "SlideKickJumping"

func enter(player):
	.enter(player)
	player.set_shape_to_ducking()
	_speed = player.horizontal_speed * player.walking_multiplier * player.running_multiplier * SPEED_MODIFIER
	_initial_speed = _speed
	_average_y_speed = player.jump_height / 10.0
	_slide_kick_weapon = SlideKick.instance()
	var pos = _slide_kick_weapon.get_pos()
	if player.direction == -1:
		_slide_kick_weapon.set_pos(Vector2(-1 * pos.x, pos.y))
	player.add_child(_slide_kick_weapon)
	_slide_kick_weapon.activate()

func _input(event):
	# Double slide kick jump.
	if player.has_extra_jump and player.should_jump(event) and not _has_landed:
		player.has_extra_jump = false
		player.get_tree().set_input_as_handled()
		_rising = true
		_timer = 0

func _fixed_process(delta):
	var fraction_of_pi_over_two = (_timer / MAX_DURATION) * PI_OVER_TWO
	var y_velocity = null
	# Go up for a while.
	if _rising:
		y_velocity = (player.jump_height * (delta / MAX_DURATION)) * (1 - sin(fraction_of_pi_over_two))
		# Stop going up when jump is released.
		if not Input.is_action_pressed("player_jump"):
			_rising = false
			_timer = 0
	# Then go down indefinitely.
	else:
		y_velocity = (player.jump_height * (delta / MAX_DURATION)) * (-1 * sin(fraction_of_pi_over_two))
	# Increase the timer, to an extent.
	if _timer < MAX_DURATION:
		_timer = _timer + delta
		# Stop rising, start falling.
		if _rising and _timer >= MAX_DURATION:
			_rising = false
			_timer = 0
		elif _timer > MAX_DURATION:
			_timer = MAX_DURATION
	# Splashdown.
	if not _rising and player.is_in_water():
		return Floating.new()
	# Slide to a stop if we land.
	if not _rising and player.is_grounded():
		_has_landed = true
		_speed = lerp(_initial_speed, 0, _stop_timer / DECELERATION_DURATION)
		_stop_timer = _stop_timer + delta
		if _stop_timer > DECELERATION_DURATION:
			_stop_timer = DECELERATION_DURATION
	# If we slide off a cliff, fall.
	if _has_landed and not player.is_grounded():
		return Falling.new()
	# Stop when we're stopped.
	if _speed == 0:
		if Input.is_action_pressed("player_duck"):
			return Ducking.new()
		return Standing.new()
	# Stop slide-kick-jumping if we hit a wall.
	if player.is_face_checking_a_wall():
		return Falling.new()
	# Move the player horizontally at high speed.
	player.move_by(Vector2(_speed * player.direction * delta, y_velocity))

func exit():
	player.set_shape_to_default()
	player.remove_child(_slide_kick_weapon)
