extends "res://Actors/Player/States/PlayerState.gd"

const Ducking = preload("res://Actors/Player/States/Ducking.gd")
const SlideKick = preload("res://Actors/Player/Weapons/SlideKick/SlideKick.tscn")
const SlideKickJumping = preload("res://Actors/Player/States/SlideKickJumping.gd")
const Standing = preload("res://Actors/Player/States/Standing.gd")

const SLIDE_KICK_DURATION = 0.5 # s
const SLIDE_KICKING_SPEED_MODIFIER = 1.75

var _slide_kick_weapon = null
var _slide_kick_time = 0
var _x_velocity = null

func get_name():
	return "SlideKicking"

func enter(player):
	.enter(player)
	player.set_shape_to_ducking()
	_x_velocity = player.horizontal_speed * player.walking_multiplier * player.running_multiplier * player.direction * SLIDE_KICKING_SPEED_MODIFIER
	_slide_kick_weapon = SlideKick.instance()
	var pos = _slide_kick_weapon.get_pos()
	if player.direction == -1:
		_slide_kick_weapon.set_pos(Vector2(-1 * pos.x, pos.y))
	player.add_child(_slide_kick_weapon)
	_slide_kick_weapon.activate()

func _input(event):
	# Slide-kick-jump.
	if player.should_jump(event):
		player.get_tree().set_input_as_handled()
		return SlideKickJumping.new()

func _fixed_process(delta):
	# Stop slide-kicking if we hit a wall.
	if player.is_face_checking_a_wall():
		if Input.is_action_pressed("player_duck"):
			return Ducking.new()
		return Standing.new()
	var y_velocity = 0
	var x_velocity = _x_velocity * delta
	# Either stick to the ground or shoot off at a funny angle, but don't fall during a slide kick.
	if not player.is_grounded():
		y_velocity = abs(x_velocity)
	# End the slide-kick prematurely if the player lets up.
	if not Input.is_action_pressed("player_duck") and y_velocity == 0:
		return Standing.new()
	# End the slide kick after its time has passed, unless we've determined that we're hovering.
	if _slide_kick_time > SLIDE_KICK_DURATION and y_velocity == 0:
		if Input.is_action_pressed("player_duck"):
			return Ducking.new()
		return Standing.new()
	# Move the player horizontally at high speed.
	player.move_by(Vector2(x_velocity, y_velocity))
	_slide_kick_time = _slide_kick_time + delta

func exit():
	player.set_shape_to_default()
	player.remove_child(_slide_kick_weapon)
