extends "res://Actors/Player/States/PlayerState.gd"

const Falling = preload("res://Actors/Player/States/Falling.gd")

const CREST_HEIGHT_FRACTION = 0.25
const CREST_TIME = 0.05 # s
const JUMP_HEIGHT_FRACTION = 0.5
const JUMP_TIME = 0.45 # s

var direction = 0

var _jump_time_remaining = JUMP_TIME
var _x_velocity = null
var _max_delta_x_velocity = null
var _target_agree_x_velocity = null

func get_name():
	return "WallJumping"

func enter(player):
	# Direction must be set by instantiator in advance.
	assert(direction != 0)
	.enter(player)
	# Keep track of simulated acceleration bounds.
	_max_delta_x_velocity = 2 * player.get_horizontal_movement_speed()
	_target_agree_x_velocity = player.get_horizontal_movement_speed() * direction * player.running_multiplier
	_x_velocity = _target_agree_x_velocity
	# Reset double-jumps.
	player.has_extra_jump = player.can_double_jump

# Go upward mostly linearly until the crest.
func _fixed_process(delta):
	# Determine slice of motion based on jump_height and time fraction.
	var y_velocity = player.jump_height * JUMP_HEIGHT_FRACTION * (delta / JUMP_TIME)
	# Reduced percentage for slightly rounded transitions.
	if _jump_time_remaining <= 0:
		y_velocity = y_velocity * CREST_HEIGHT_FRACTION * ((CREST_TIME + _jump_time_remaining) / CREST_TIME)
	if y_velocity > 0:
		return Falling.new()
	_jump_time_remaining = _jump_time_remaining - delta
	# Determine X velocity by stepping toward double speed, stopped, or negative speed, dependent on input.
	var step = (_max_delta_x_velocity * delta) / (JUMP_TIME + CREST_TIME)
	var last_x_velocity = _x_velocity
	# Accelerate or maintain max speed.
	if _directions_agree():
		_x_velocity = _x_velocity + (step * direction)
		if abs(_x_velocity) > abs(_target_agree_x_velocity):
			_x_velocity = _target_agree_x_velocity
	# Otherwise slow to a stop.
	else:
		if _x_velocity > 0:
			_x_velocity = _x_velocity - step
			if _x_velocity < 0:
				_x_velocity = 0
		if _x_velocity < 0:
			_x_velocity = _x_velocity + step
			if _x_velocity > 0:
				_x_velocity = 0
	player.move_by(Vector2(_x_velocity * delta, y_velocity))

# Return true iff the curent held input agrees with the stored direction.
func _directions_agree():
	if Input.is_action_pressed("player_right") and direction == 1:
		return true
	if Input.is_action_pressed("player_left") and direction == -1:
		return true
	return false
