extends "res://Actors/Player/Weapons/Weapon.gd"

const Projectile = preload("res://Actors/Player/Weapons/Grappler/Projectile.tscn")

const COLOR = Color(1, 0, 0)
const SPEED = 500.0
const WIDTH = 3.0

var _initial_position = null
var _max_distance = 250
var _projectile_ref = null

onready var _level = Global.get_level()
onready var _level_canvas_rid = _level.get_canvas_item()
onready var _torso = get_node("../../..")

# Spawn a projectile and shoot it off at the appropriate angle.
func shoot():
	_initial_position = get_global_pos()
	var projectile = Projectile.instance()
	_projectile_ref = weakref(projectile)
	projectile.set_pos(_initial_position)
	_level.add_child(projectile)
	var relative_vector = _initial_position - _torso.get_global_pos()
	var angle = atan2(-relative_vector.y, relative_vector.x)
	projectile.set_linear_velocity(Vector2(SPEED * cos(angle), -SPEED * sin(angle)))
	projectile.set_fixed_process(true)
	set_fixed_process(true)
	set_process(true)

# Keep track of the state of the projectile.
func _fixed_process(delta):
	# Kill the projectile if it goes too far.
	var projectile = _projectile_ref.get_ref()
	if projectile != null and (projectile.get_global_pos() - _initial_position).length() > _max_distance:
		projectile.queue_free()
		set_fixed_process(false)
		set_process(false)
		VisualServer.canvas_item_clear(_level_canvas_rid)
		return

# Draw a line between the projectile and the grappler.
func _process(delta):
	var projectile = _projectile_ref.get_ref()
	if projectile != null:
		VisualServer.canvas_item_clear(_level_canvas_rid)
		VisualServer.canvas_item_add_line(_level_canvas_rid, get_global_pos(), projectile.get_global_pos(), COLOR, WIDTH)
	else:
		set_process(false)
		set_fixed_process(false)
