# Superclass (faux interface) common to all finite state machine / pushdown automata.

var parent = null

# Return the unique string name of the state. Must be overridden.
func get_name():
	assert(false)

# Handle any transitions into this state. Subclasses should first chain to this method.
func enter(parent):
	self.parent = parent
	_animate()

# Transition to a new animation; by default, one matching the name of the State (if it exists).
# Can be overridden without chaining.
func _animate():
	var animation_player = parent.get_node("AnimationPlayer")
	if animation_player == null:
		return
	var name = get_name()
	if animation_player.has_animation(name):
		animation_player.play(name)

# Handle input events.
func _input(event):
	pass

# Update physics processing.
func _fixed_process(delta):
	pass

# Update regular processing.
func _process(delta):
	pass

# Handle exit events.
func exit():
	pass

# Call the given function with the given arg and iterate if state changes.
func state_loop(function, arg):
	# Keep a list of old states to prevent cycles.
	var old_states = []
	var new_state = parent.state.call(function, arg)
	while new_state != null:
		var new_state_name = new_state.get_name()
		# Throw an exception if we re-enter a previously visited state this frame.
		assert(old_states.find(new_state_name) == -1)
		old_states.append(new_state_name)
		set_state(new_state)
		# Let our new state run this cycle since our old state ended.
		new_state = parent.state.call(function, arg)

# Exit the current state, enter the new state.
func set_state(state):
	parent.state.exit()
	parent.state = state
	state.enter(parent)
