extends RigidBody2D

const Player = preload("res://Actors/Player/Player.gd")
const Enemy = preload("res://Actors/Enemies/Enemy.gd")

const FADE_IN_TIME = 0.25 # s

export(String) var name = null
export(String) var description = null

export(float) var lifetime = null
export(int, "Comestible") var category = null

var _half_life = null
var _timer = 0

# TODO: Spawn increasingly helpful items after consecutive deaths without healing.

# Start fadeout timer and connect collision function.
func _ready():
	assert(lifetime != null)
	assert(name != null)
	assert(description != null)
	assert(category != null)
	_half_life = lifetime / 2.0
	set_layer_mask(0)
	set_collision_mask(2)
	var area = get_node("Area2D")
	area.connect("area_enter", self, "_on_area_enter")
	area.set_layer_mask(3)
	area.set_collision_mask(3)
	if lifetime > 0:
		set_opacity(0)
		set_process(true)

# Only get collected by CollectsItems members.
func _on_area_enter(area):
	if area.is_in_group("CollectsItems"):
		# Don't get collected on fade-in.
		if lifetime > 0 and get_opacity() < 1:
			return
		collect(area)

# What do do if a CollectsItems area enters our area.
# Should be overridden (but chained to) by subclasses.
func collect(collector):
	if collector.get_node("..") == Global.get_player():
		Inventory.add(self)
	queue_free()

# Fade in, fade out, self-destruct.
func _process(delta):
	# Fade in over a brief period of time.
	if _timer < FADE_IN_TIME:
		var percent = _timer + delta
		if percent >= FADE_IN_TIME:
			percent = FADE_IN_TIME
			# Perhaps get collected immediately upon fade-in completion.
			for area in get_node("Area2D").get_overlapping_areas():
				if area.is_in_group("CollectsItems"):
					collect(area)
					return
		percent = percent / FADE_IN_TIME
		set_opacity(percent)
	_timer = _timer + delta
	if _timer >= lifetime:
		queue_free()
	# Fade out after a while.
	if _timer > _half_life:
		set_opacity(1 - ((_timer - _half_life) / _half_life))
