extends Node2D

const DEBUG = true

export(int) var camera_margin_left = null
export(int) var camera_margin_top = null
export(int) var camera_margin_right = null
export(int) var camera_margin_bottom = null

var debug_rect = null setget _set_debug_rect

func _init():
	add_to_group("Level")

func _ready():
	_clamp_camera()
	_add_layer_to_static_children(self)
	_ensure_hud_exists()
	if DEBUG == true:
		get_node("HUD").add_child(load("res://Debug/DebugPanel/DebugPanel.tscn").instance())

# Set the camera's limits.
func _clamp_camera():
	assert(camera_margin_left != null)
	assert(camera_margin_top != null)
	assert(camera_margin_right != null)
	assert(camera_margin_bottom != null)
	var camera = Global.get_player().get_node("Camera2D")
	camera.set_limit(MARGIN_LEFT, camera_margin_left)
	camera.set_limit(MARGIN_TOP, camera_margin_top)
	camera.set_limit(MARGIN_RIGHT, camera_margin_right)
	camera.set_limit(MARGIN_BOTTOM, camera_margin_bottom)

# Ensure HUD layer exists.
func _ensure_hud_exists():
	var make_hud = true
	for child in get_children():
		if child.get_name() == "HUD":
			make_hud = false
			break
	if make_hud == true:
		var hud = CanvasLayer.new()
		hud.set_layer(1)
		hud.set_name("HUD")
		add_child(hud)

# Add Layer 1 to all StaticBody2Ds so Enemies can interact with the environment but clip through the Player.
func _add_layer_to_static_children(body):
	if body extends StaticBody2D:
		body.set_collision_mask_bit(1, true)
		body.set_layer_mask_bit(1, true)
	for child in body.get_children():
		_add_layer_to_static_children(child)

func _draw():
	if DEBUG == true and debug_rect != null and OS.is_debug_build():
		draw_rect(debug_rect, Color(1, 1, 1))

func _set_debug_rect(rect):
	debug_rect = rect
	update()
